# Generated by roxygen2: do not edit by hand

S3method(as_osm_url_spec,character)
S3method(as_osm_url_spec,osm_url_spec)
export(as.tile_source)
export(as_osm_url_spec)
export(bmaps.plot)
export(bmaps.types)
export(extract_bbox)
export(get_default_cachedir)
export(get_default_tile_source)
export(has_internet)
export(is.tile_source)
export(makebbox)
export(osm.image)
export(osm.lines)
export(osm.plot)
export(osm.points)
export(osm.polygon)
export(osm.raster)
export(osm.segments)
export(osm.text)
export(osm.types)
export(osm_crs_native)
export(osm_ensure_lnglat)
export(osm_ensure_native)
export(osm_lnglat)
export(osm_native)
export(osm_raster)
export(osm_tile)
export(osm_tile_covering)
export(osm_tile_envelope)
export(osm_tile_quadkey)
export(osm_tile_top_left)
export(osm_url)
export(osm_url_load_async)
export(osm_url_spec)
export(osm_url_spec_example)
export(osm_zoom_num_tiles)
export(register_tile_source)
export(set_default_cachedir)
export(set_default_tile_source)
export(source_from_url_format)
export(zoombbox)
